#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_controllers_anchor_capable_devices_info
short_description: Information module for Wireless Controllers
  Anchor Capable Devices
description:
  - Get all Wireless Controllers Anchor Capable Devices.
  - This API allows the user to get Anchor capable devices.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Wireless
      GetAnchorCapableDevices
    description: Complete reference of the GetAnchorCapableDevices
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-anchor-capable-devices
notes:
  - SDK Method used are
    wireless.Wireless.get_anchor_capable_devices,
  - Paths used are
    get /dna/intent/api/v1/wirelessControllers/anchorCapableDevices,
"""

EXAMPLES = r"""
---
- name: Get all Wireless Controllers Anchor Capable
    Devices
  cisco.dnac.wireless_controllers_anchor_capable_devices_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "deviceIp": "string",
      "deviceName": "string",
      "wirelessMgmtIP": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: floors_floor_id_planned_access_point_positions_id
short_description: Resource module for Floors Floor
  Id Planned Access Point Positions Id
description:
  - Manage operation delete of the resource Floors Floor
    Id Planned Access Point Positions Id.
  - Delete specified Planned Access Points Position
    designated for a specific floor.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  floorId:
    description: FloorId path parameter. Floor Id.
    type: str
  id:
    description: Id path parameter. Planned Access Point
      Id.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Site Design
      DeletePlannedAccessPointsPositionV2
    description: Complete reference of the DeletePlannedAccessPointsPositionV2
      API.
    link: https://developer.cisco.com/docs/dna-center/#!delete-planned-access-points-position-v-2
notes:
  - SDK Method used are
    site_design.SiteDesign.delete_planned_access_points_position_v2,
  - Paths used are
    delete /dna/intent/api/v2/floors/{floorId}/plannedAccessPointPositions/{id},
"""

EXAMPLES = r"""
---
- name: Delete by id
  cisco.dnac.floors_floor_id_planned_access_point_positions_id:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    floorId: string
    id: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_live_tools_leds_blink
short_description: Resource module for devices _live _tools _leds _blink
description:
  - Manage operation create of the resource devices _live _tools _leds _blink.
  - Enqueue a job to blink LEDs on a device. This endpoint has a rate limit of one request every 10 seconds.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  callback:
    description: Details for the callback. Please include either an httpServerId OR url and sharedSecret.
    suboptions:
      httpServer:
        description: The webhook receiver used for the callback webhook.
        suboptions:
          id:
            description: The webhook receiver ID that will receive information. If specifying this, please leave the url and sharedSecret fields
              blank.
            type: str
        type: dict
      payloadTemplate:
        description: The payload template of the webhook used for the callback.
        suboptions:
          id:
            description: The ID of the payload template. Defaults to 'wpt_00005' for the Callback (included) template.
            type: str
        type: dict
      sharedSecret:
        description: A shared secret that will be included in the requests sent to the callback URL. It can be used to verify that the request
          was sent by Meraki. If using this field, please also specify an url.
        type: str
      url:
        description: The callback URL for the webhook target. If using this field, please also specify a sharedSecret.
        type: str
    type: dict
  duration:
    description: The duration in seconds to blink LEDs.
    type: int
  serial:
    description: Serial path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for devices createDeviceLiveToolsLedsBlink
    description: Complete reference of the createDeviceLiveToolsLedsBlink API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-device-live-tools-leds-blink
notes:
  - SDK Method used are
    devices.Devices.create_device_live_tools_leds_blink,
  - Paths used are
    post /devices/{serial}/liveTools/leds/blink,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.devices_live_tools_leds_blink:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    callback:
      httpServer:
        id: aHR0cHM6Ly93d3cuZXhhbXBsZS5jb20vd2ViaG9va3M=
      payloadTemplate:
        id: wpt_2100
      sharedSecret: secret
      url: https://webhook.site/28efa24e-f830-4d9f-a12b-fbb9e5035031
    duration: 30
    serial: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "callback": {
        "id": "string",
        "status": "string",
        "url": "string"
      },
      "error": "string",
      "ledsBlinkId": "string",
      "request": {
        "duration": 0,
        "serial": "string"
      },
      "status": "string",
      "url": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: application_visibility_network_devices_enable_cbar
short_description: Resource module for Application Visibility
  Network Devices Enable Cbar
description:
  - Manage operation create of the resource Application
    Visibility Network Devices Enable Cbar. - > This
    API can be used to enable CBAR feature on multiple
    network devices. Request payload should include
    the list of network devices where CBAR has to be
    enabled. It can optionally include list of interfaces
    wired or WLAN modes wireless to be excluded from
    CBAR enablement.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  networkDevices:
    description: Application Visibility Network Devices
      Enable Cbar's networkDevices.
    elements: dict
    suboptions:
      excludeInterfaceIds:
        description: List of interface identifiers which
          needs to be excluded from CBAR enablement.
          Applicable only for wired devices. Please
          note that this list considered as absolute
          exclusion and earlier exclusions are not considered.
          For example, if IF1 and IF2 have already been
          excluded from CBAR as part of earlier enablement,
          and this API is now called with IF3 and IF4
          as inputs, then IF1 and IF2 are removed from
          exclusion list and only IF3 and IF4 are excluded.
        elements: str
        type: list
      excludeWlanModes:
        description: WLAN modes which needs to be excluded
          from CBAR enablement. Applicable only for
          wireless devices. Applicable values are LOCAL,
          FLEX, or FABRIC.
        elements: str
        type: list
      id:
        description: Network device identifier.
        type: str
    type: list
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Application
      Policy EnableCBARFeatureOnMultipleNetworkDevices
    description: Complete reference of the EnableCBARFeatureOnMultipleNetworkDevices
      API.
    link: https://developer.cisco.com/docs/dna-center/#!enable-cbar-feature-on-multiple-network-devices
notes:
  - SDK Method used are
    application_policy.ApplicationPolicy.enable_c_b_a_r_feature_on_multiple_network_devices,
  - Paths used are
    post /dna/intent/api/v1/applicationVisibility/networkDevices/enableCbar,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.application_visibility_network_devices_enable_cbar:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    networkDevices:
      - excludeInterfaceIds:
          - string
        excludeWlanModes:
          - string
        id: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: aaa_services_query_count
short_description: Resource module for Aaa Services
  Query Count
description:
  - Manage operation create of the resource Aaa Services
    Query Count. - > Retrieves the total number of AAA
    Services and offers complex filtering and sorting
    capabilities. For detailed information about the
    usage of the API, please refer to the Open API specification
    document - https //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
    AAAServices-1.0.0-resolved.yaml.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  endTime:
    description: End Time.
    type: int
  filters:
    description: Aaa Services Query Count's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        elements: str
        type: list
    type: list
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description: Start Time.
    type: int
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Devices
      RetrievesTheTotalNumberOfAAAServicesForGivenSetOfComplexFilters
    description: Complete reference of the RetrievesTheTotalNumberOfAAAServicesForGivenSetOfComplexFilters
      API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieves-the-total-number-of-aaa-services-for-given-set-of-complex-filters
notes:
  - SDK Method used are
    devices.Devices.retrieves_the_total_number_of_aaa_services_for_given_set_of_complex_filters,
  - Paths used are
    post /dna/data/api/v1/aaaServices/query/count,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.aaa_services_query_count:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    endTime: 0
    filters:
      - key: string
        operator: string
        value:
          - string
    headers: '{{my_headers | from_json}}'
    startTime: 0
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: custom_issue_definitions_count_info
short_description: Information module for Custom Issue
  Definitions Count
description:
  - Get all Custom Issue Definitions Count. - > Get
    the total number of Custom issue definitions count
    based on the provided filters. The supported filters
    are id, name, profileId and definition enable status,
    severity, facility and mnemonic. For detailed information
    about the usage of the API, please refer to the
    Open API specification document - https //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
    AssuranceUserDefinedIssueAPIs-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
      - >
        Id query parameter. The custom issue definition
        identifier and unique identifier across the
        profile. Examples id=6bef213c-19ca-4170-8375-b694e251101c
        (single entity uuid requested) id=6bef213c-19ca-4170-8375-b694e251101c&id=19ca-4170-8375-b694e251101c-6bef213c
        (multiple Id request in the query param).
    type: str
  profileId:
    description:
      - >
        ProfileId query parameter. The profile identifier
        to fetch the profile associated custom issue
        definitions. The default is global. For the
        custom profile, it is profile UUID. Example
        3fa85f64-5717-4562-b3fc-2c963f66afa6.
    type: str
  name:
    description:
      - Name query parameter. The list of UDI issue
        names. (Ex."TestUdiIssues").
    type: str
  priority:
    description:
      - >
        Priority query parameter. The Issue priority
        value, possible values are P1, P2, P3, P4. P1
        A critical issue that needs immediate attention
        and can have a wide impact on network operations.
        P2 A major issue that can potentially impact
        multiple devices or clients. P3 A minor issue
        that has a localized or minimal impact. P4 A
        warning issue that may not be an immediate problem
        but addressing it can optimize the network performance.
    type: str
  isEnabled:
    description:
      - IsEnabled query parameter. The enable status
        of the custom issue definition, either true
        or false.
    type: bool
  severity:
    description:
      - >
        Severity query parameter. The syslog severity
        level. 0 Emergency 1 Alert, 2 Critical. 3 Error,
        4 Warning, 5 Notice, 6 Info. Examples severity=1&severity=2
        (multi value support with & separator).
    type: float
  facility:
    description:
      - Facility query parameter. The syslog facility
        name.
    type: str
  mnemonic:
    description:
      - Mnemonic query parameter. The syslog mnemonic
        name.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Issues
      GetTheTotalCustomIssueDefinitionsCountBasedOnTheProvidedFilters
    description: Complete reference of the GetTheTotalCustomIssueDefinitionsCountBasedOnTheProvidedFilters
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-the-total-custom-issue-definitions-count-based-on-the-provided-filters
notes:
  - SDK Method used are
    issues.Issues.get_the_total_custom_issue_definitions_count_based_on_the_provided_filters,
  - Paths used are
    get /dna/intent/api/v1/customIssueDefinitions/count,
"""

EXAMPLES = r"""
---
- name: Get all Custom Issue Definitions Count
  cisco.dnac.custom_issue_definitions_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    profileId: string
    name: string
    priority: string
    isEnabled: true
    severity: 0
    facility: string
    mnemonic: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_assurance_alerts_overview_by_network_info
short_description: Information module for organizations _assurance _alerts _overview _by _network
description:
  - Get all organizations _assurance _alerts _overview _by _network.
  - Return a Summary of Alerts grouped by network and severity.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the start of the page. Often this is a timestamp or an ID but it
        is not limited to those. This parameter should not be defined by client applications. The link for the first, last, prev, or next page
        in the HTTP Link header should define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end of the page. Often this is a timestamp or an ID but it is
        not limited to those. This parameter should not be defined by client applications. The link for the first, last, prev, or next page in
        the HTTP Link header should define it.
    type: str
  sortOrder:
    description:
      - >
        SortOrder query parameter. Sorted order of entries. Order options are 'ascending' and 'descending'. Default is 'ascending'.
    type: str
  networkId:
    description:
      - NetworkId query parameter. Optional parameter to filter alerts overview by network id.
    type: str
  severity:
    description:
      - Severity query parameter. Optional parameter to filter alerts overview by severity type.
    type: str
  types:
    description:
      - Types query parameter. Optional parameter to filter by alert type.
    elements: str
    type: list
  tsStart:
    description:
      - TsStart query parameter. Optional parameter to filter by starting timestamp.
    type: str
  tsEnd:
    description:
      - TsEnd query parameter. Optional parameter to filter by end timestamp.
    type: str
  category:
    description:
      - Category query parameter. Optional parameter to filter by category.
    type: str
  serials:
    description:
      - Serials query parameter. Optional parameter to filter by primary device serial.
    elements: str
    type: list
  deviceTypes:
    description:
      - DeviceTypes query parameter. Optional parameter to filter by device types.
    elements: str
    type: list
  deviceTags:
    description:
      - DeviceTags query parameter. Optional parameter to filter by device tags.
    elements: str
    type: list
  active:
    description:
      - Active query parameter. Optional parameter to filter by active alerts defaults to true.
    type: bool
  dismissed:
    description:
      - Dismissed query parameter. Optional parameter to filter by dismissed alerts defaults to false.
    type: bool
  resolved:
    description:
      - Resolved query parameter. Optional parameter to filter by resolved alerts defaults to false.
    type: bool
  suppressAlertsForOfflineNodes:
    description:
      - >
        SuppressAlertsForOfflineNodes query parameter. When set to true the api will only return connectivity alerts for a given device if that
        device is in an offline state. This only applies to devices. This is ignored when resolved is true. Example If a Switch has a VLan Mismatch
        and is Unreachable. Only the Unreachable alert will be returned. Defaults to false.
    type: bool
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationAssuranceAlertsOverviewByNetwork
    description: Complete reference of the getOrganizationAssuranceAlertsOverviewByNetwork API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-assurance-alerts-overview-by-network
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_assurance_alerts_overview_by_network,
  - Paths used are
    get /organizations/{organizationId}/assurance/alerts/overview/byNetwork,
"""

EXAMPLES = r"""
- name: Get all organizations _assurance _alerts _overview _by _network
  cisco.meraki.organizations_assurance_alerts_overview_by_network_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    sortOrder: string
    networkId: string
    severity: string
    types: []
    tsStart: string
    tsEnd: string
    category: string
    serials: []
    deviceTypes: []
    deviceTags: []
    active: true
    dismissed: true
    resolved: true
    suppressAlertsForOfflineNodes: true
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "alertCount": 0,
          "networkId": "string",
          "networkName": "string",
          "severityCounts": [
            {
              "count": 0,
              "type": "string"
            }
          ]
        }
      ],
      "meta": {
        "counts": {
          "items": 0
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: integration_settings_instances_itsm_info
short_description: Information module for Integration
  Settings Instances Itsm
description:
  - Get Integration Settings Instances Itsm by id.
  - Fetches ITSM Integration setting by ID.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  instanceId:
    description:
      - InstanceId path parameter. Instance Id of the
        Integration setting instance.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for ITSM Integration
      GetITSMIntegrationSettingById
    description: Complete reference of the GetITSMIntegrationSettingById
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-itsm-integration-setting-by-id
notes:
  - SDK Method used are
    itsm_integration.ItsmIntegration.get_itsm_integration_setting_by_id,
  - Paths used are
    get /dna/intent/api/v1/integration-settings/instances/itsm/{instanceId},
"""

EXAMPLES = r"""
---
- name: Get Integration Settings Instances Itsm by id
  cisco.dnac.integration_settings_instances_itsm_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    instanceId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "_id": "string",
      "id": "string",
      "dypId": "string",
      "dypName": "string",
      "dypMajorVersion": 0,
      "name": "string",
      "uniqueKey": "string",
      "description": "string",
      "data": {
        "ConnectionSettings": {
          "Url": "string",
          "Auth_UserName": "string",
          "Auth_Password": "string"
        }
      },
      "updatedDate": 0,
      "updatedBy": "string",
      "tenantId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: custom_issue_definitions
short_description: Resource module for Custom Issue
  Definitions
description:
  - Manage operations create, update and delete of the
    resource Custom Issue Definitions. - > Create a
    new custom issue definition using the provided input
    request data. The unique identifier for this issue
    definition is id. Please note that the issue names
    cannot be duplicated. The definition is based on
    the syslog. For detailed information about the usage
    of the API, please refer to the Open API specification
    document - https //github.com/cisco-en-programmability/catalyst-center-api-
    specs/blob/main/Assurance/CE_Cat_Center_Org-AssuranceUserDefinedIssueAPIs-1.0.0-resolved.yaml.
    - > Deletes an existing custom issue definition
    based on the Id. Only the Global profile issue has
    the access to delete the issue definition, so no
    profile id is required. For detailed information
    about the usage of the API, please refer to the
    Open API specification document - https //github.com/cisco-en-
    programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
    AssuranceUserDefinedIssueAPIs-1.0.0-resolved.yaml.
    - > Updates an existing custom issue definition
    based on the provided Id. For detailed information
    about the usage of the API, please refer to the
    Open API specification document - https //github.com/cisco-en-
    programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
    AssuranceUserDefinedIssueAPIs-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  description:
    description: Description.
    type: str
  headers:
    description: Additional headers.
    type: dict
  id:
    description: Id path parameter. The custom issue
      definition Identifier.
    type: str
  isEnabled:
    description: Is Enabled.
    type: bool
  isNotificationEnabled:
    description: Is Notification Enabled.
    type: bool
  name:
    description: Name.
    type: str
  priority:
    description: Priority.
    type: str
  rules:
    description: Custom Issue Definitions's rules.
    elements: dict
    suboptions:
      durationInMinutes:
        description: Duration In Minutes.
        type: int
      facility:
        description: Facility.
        type: str
      mnemonic:
        description: Mnemonic.
        type: str
      occurrences:
        description: Occurrences.
        type: int
      pattern:
        description: Pattern.
        type: str
      severity:
        description: Severity.
        type: int
    type: list
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Issues
      CreatesANewUserDefinedIssueDefinitions
    description: Complete reference of the CreatesANewUserDefinedIssueDefinitions
      API.
    link: https://developer.cisco.com/docs/dna-center/#!creates-a-new-user-defined-issue-definitions
  - name: Cisco DNA Center documentation for Issues
      DeletesAnExistingCustomIssueDefinition
    description: Complete reference of the DeletesAnExistingCustomIssueDefinition
      API.
    link: https://developer.cisco.com/docs/dna-center/#!deletes-an-existing-custom-issue-definition
  - name: Cisco DNA Center documentation for Issues
      UpdatesAnExistingCustomIssueDefinitionBasedOnTheProvidedId
    description: Complete reference of the UpdatesAnExistingCustomIssueDefinitionBasedOnTheProvidedId
      API.
    link: https://developer.cisco.com/docs/dna-center/#!updates-an-existing-custom-issue-definition-based-on-the-provided-id
notes:
  - SDK Method used are
    issues.Issues.creates_a_new_user_defined_issue_definitions,
    issues.Issues.deletes_an_existing_custom_issue_definition,
    issues.Issues.updates_an_existing_custom_issue_definition_based_on_the_provided_id,
  - Paths used are
    post /dna/intent/api/v1/customIssueDefinitions,
    delete /dna/intent/api/v1/customIssueDefinitions/{id},
    put /dna/intent/api/v1/customIssueDefinitions/{id},
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.custom_issue_definitions:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    headers: '{{my_headers | from_json}}'
    isEnabled: true
    isNotificationEnabled: true
    name: string
    priority: string
    rules:
      - durationInMinutes: 0
        facility: string
        mnemonic: string
        occurrences: 0
        pattern: string
        severity: 0
- name: Update by id
  cisco.dnac.custom_issue_definitions:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    id: string
    isEnabled: true
    isNotificationEnabled: true
    name: string
    priority: string
    rules:
      - durationInMinutes: 0
        facility: string
        mnemonic: string
        occurrences: 0
        pattern: string
        severity: 0
- name: Delete by id
  cisco.dnac.custom_issue_definitions:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "name": "string",
        "description": "string",
        "profileId": "string",
        "triggerId": "string",
        "rules": [
          {
            "type": "string",
            "severity": 0,
            "facility": "string",
            "mnemonic": "string",
            "pattern": "string",
            "occurrences": 0,
            "durationInMinutes": 0
          }
        ],
        "isEnabled": true,
        "priority": "string",
        "isDeletable": true,
        "isNotificationEnabled": true,
        "createdTime": 0,
        "lastUpdatedTime": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_bugs_results_bugs_id_network_devices_network_device_id_info
short_description: Information module for Network Bugs
  Results Bugs Id Network Devices Network Device Id
description:
  - Get Network Bugs Results Bugs Id Network Devices
    Network Device Id by id.
  - Get network bug device for the bug by network device
    id.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
      - Id path parameter. Id of the network bug.
    type: str
  networkDeviceId:
    description:
      - NetworkDeviceId path parameter. Id of the network
        device.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Compliance
      GetNetworkBugDeviceForTheBugByNetworkDeviceId
    description: Complete reference of the GetNetworkBugDeviceForTheBugByNetworkDeviceId
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-network-bug-device-for-the-bug-by-network-device-id
notes:
  - SDK Method used are
    compliance.Compliance.get_network_bug_device_for_the_bug_by_network_device_id,
  - Paths used are
    get /dna/intent/api/v1/networkBugs/results/bugs/{id}/networkDevices/{networkDeviceId},
"""

EXAMPLES = r"""
---
- name: Get Network Bugs Results Bugs Id Network Devices
    Network Device Id by id
  cisco.dnac.network_bugs_results_bugs_id_network_devices_network_device_id_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    networkDeviceId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "response": {
          "networkDeviceId": "string",
          "bugCount": 0,
          "scanMode": "string",
          "scanStatus": "string",
          "comments": "string",
          "lastScanTime": 0
        },
        "version": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_settings_ap_profiles
short_description: Resource module for Wireless Settings
  Ap Profiles
description:
  - Manage operation create of the resource Wireless
    Settings Ap Profiles.
  - This API allows the user to create a custom AP Profile.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  apPowerProfileName:
    description: Name of the existing AP power profile.
    type: str
  apProfileName:
    description: Name of the Access Point profile. Max
      length is 32 characters.
    type: str
  awipsEnabled:
    description: Indicates if AWIPS is enabled on the
      AP.
    type: bool
  awipsForensicEnabled:
    description: Indicates if AWIPS forensic is enabled
      on the AP. Forensic Capture is supported from
      IOS-XE version 17.4 and above. Forensic Capture
      can be activated only if aWIPS is enabled.
    type: bool
  calendarPowerProfiles:
    description: Wireless Settings Ap Profiles's calendarPowerProfiles.
    suboptions:
      duration:
        description: Wireless Settings Ap Profiles's
          duration.
        suboptions:
          schedulerDate:
            description: Start and End date of the duration
              setting, applicable for MONTHLY schedulers.
              Values must be between 1 and 31.
            elements: str
            type: list
          schedulerDay:
            description: Applies every week on the selected
              days. Ex "sunday","saturday","tuesday","wednesday","thu...
            elements: str
            type: list
          schedulerEndTime:
            description: End time of the duration setting.
            type: str
          schedulerStartTime:
            description: Start time of the duration
              setting.
            type: str
        type: dict
      powerProfileName:
        description: Name of the existing AP power profile
          to be mapped to the calendar power profile.
          API-/intent/api/v1/wirelessSettings/powerProfiles.
        type: str
      schedulerType:
        description: Type of the scheduler.
        type: str
    type: dict
  clientLimit:
    description: Number of clients. Value should be
      between 0-1200.
    type: int
  countryCode:
    description: Country Code.
    type: str
  description:
    description: Description of the AP profile. Max
      length is 241 characters.
    type: str
  managementSetting:
    description: Wireless Settings Ap Profiles's managementSetting.
    suboptions:
      authType:
        description: Authentication type used in the
          AP profile. These setting are applicable during
          PnP claim and for day-N authentication of
          AP. Changing these settings will be service
          impacting for the PnP onboarded APs and will
          need a factory-reset for those APs.
        type: str
      cdpState:
        description: Indicates if CDP is enabled on
          the AP. Enable CDP in order to make Cisco
          Access Points known to its neighboring devices
          and vice-versa.
        type: bool
      dot1xPassword:
        description: Password for 802.1X authentication.
          Length must be 8-120 characters.
        type: str
      dot1xUsername:
        description: Username for 802.1X authentication.
          Dot1xUsername must have a minimum of 1 character
          and a maximum of 32 characters.
        type: str
      managementEnablePassword:
        description: Enable password for managing the
          AP. Length must be 8-120 characters.
        type: str
      managementPassword:
        description: Management password for the AP.
          Length must be 8-120 characters.
        type: str
      managementUserName:
        description: Management username must have a
          minimum of 1 character and a maximum of 32
          characters.
        type: str
      sshEnabled:
        description: Indicates if SSH is enabled on
          the AP. Enable SSH add credentials for device
          management.
        type: bool
      telnetEnabled:
        description: Indicates if Telnet is enabled
          on the AP. Enable Telnet to add credentials
          for device management.
        type: bool
    type: dict
  meshEnabled:
    description: This indicates whether mesh networking
      is enabled on the AP. For IOS-XE devices, when
      mesh networking is enabled, a custom mesh profile
      with the configured parameters will be created
      and mapped to the AP join profile on the device.
      When mesh networking is disabled, any existing
      custom mesh profile will be deleted from the device,
      and the AP join profile will be mapped to the
      default mesh profile on the device.
    type: bool
  meshSetting:
    description: Wireless Settings Ap Profiles's meshSetting.
    suboptions:
      backhaulClientAccess:
        description: Indicates if backhaul client access
          is enabled on the AP.
        type: bool
      bridgeGroupName:
        description: Name of the bridge group for mesh
          settings. If not configured, 'Default' Bridge
          group name will be used in mesh profile.
        type: str
      ghz24BackhaulDataRates:
        description: 2.4GHz backhaul data rates.
        type: str
      ghz5BackhaulDataRates:
        description: 5GHz backhaul data rates.
        type: str
      range:
        description: Range of the mesh network. Value
          should be between 150-132000.
        type: int
      rapDownlinkBackhaul:
        description: Type of downlink backhaul used.
        type: str
    type: dict
  pmfDenialEnabled:
    description: Indicates if PMF denial is active on
      the AP. PMF Denial is supported from IOS-XE version
      17.12 and above.
    type: bool
  remoteWorkerEnabled:
    description: Indicates if remote worker mode is
      enabled on the AP. Remote teleworker enabled profile
      cannot support security features like aWIPS,Forensic
      Capture Enablement, Rogue Detection and Rogue
      Containment.
    type: bool
  rogueDetectionSetting:
    description: Wireless Settings Ap Profiles's rogueDetectionSetting.
    suboptions:
      rogueDetection:
        description: Indicates if rogue detection is
          enabled on the AP. Detect Access Points that
          have been installed on a secure network without
          explicit authorization from a system administrator
          and configure rogue general configuration
          parameters.
        type: bool
      rogueDetectionMinRssi:
        description: Minimum RSSI for rogue detection.
          Value should be in range -128 decibel milliwatts
          and -70 decibel milliwatts.
        type: int
      rogueDetectionReportInterval:
        description: Report interval for rogue detection.
          Value should be in range 10 and 300.
        type: int
      rogueDetectionTransientInterval:
        description: Transient interval for rogue detection.
          Value should be 0 or from 120 to 1800.
        type: int
    type: dict
  timeZone:
    description: In the Time Zone area, choose one of
      the following options. Not Configured - APs operate
      in the UTC time zone. Controller - APs operate
      in the Cisco Wireless Controller time zone. Delta
      from Controller - APs operate in the offset time
      from the wireless controller time zone.
    type: str
  timeZoneOffsetHour:
    description: Enter the hour value (HH). The valid
      range is from -12 through 14.
    type: int
  timeZoneOffsetMinutes:
    description: Enter the minute value (MM). The valid
      range is from 0 through 59.
    type: int
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Wireless
      CreateAPProfile
    description: Complete reference of the CreateAPProfile
      API.
    link: https://developer.cisco.com/docs/dna-center/#!create-ap-profile
notes:
  - SDK Method used are
    wireless.Wireless.create_ap_profile,
  - Paths used are
    post /dna/intent/api/v1/wirelessSettings/apProfiles,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.wireless_settings_ap_profiles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    apPowerProfileName: string
    apProfileName: string
    awipsEnabled: true
    awipsForensicEnabled: true
    calendarPowerProfiles:
      duration:
        schedulerDate:
          - string
        schedulerDay:
          - string
        schedulerEndTime: string
        schedulerStartTime: string
      powerProfileName: string
      schedulerType: string
    clientLimit: 0
    countryCode: string
    description: string
    managementSetting:
      authType: string
      cdpState: true
      dot1xPassword: string
      dot1xUsername: string
      managementEnablePassword: string
      managementPassword: string
      managementUserName: string
      sshEnabled: true
      telnetEnabled: true
    meshEnabled: true
    meshSetting:
      backhaulClientAccess: true
      bridgeGroupName: string
      ghz24BackhaulDataRates: string
      ghz5BackhaulDataRates: string
      range: 0
      rapDownlinkBackhaul: string
    pmfDenialEnabled: true
    remoteWorkerEnabled: true
    rogueDetectionSetting:
      rogueDetection: true
      rogueDetectionMinRssi: 0
      rogueDetectionReportInterval: 0
      rogueDetectionTransientInterval: 0
    timeZone: string
    timeZoneOffsetHour: 0
    timeZoneOffsetMinutes: 0
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: icap_settings_device_deployments_info
short_description: Information module for Icap Settings
  Device Deployments
description:
  - Get all Icap Settings Device Deployments. - > Retrieves
    ICAP configuration deployment statuss per device
    based on filter criteria. For detailed information
    about the usage of the API, please refer to the
    Open API specification document - https //github.com/cisco-en-
    programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-ICAP_APIs-1.0.0-resolved.yaml.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deployActivityId:
    description:
      - DeployActivityId query parameter. Activity from
        the /deploy task response.
    type: str
  networkDeviceIds:
    description:
      - NetworkDeviceIds query parameter. Device ids,
        retrievable from the id attribute in intent/api/v1/network-device.
    type: str
  offset:
    description:
      - Offset query parameter. The first record to
        show for this page; the first record is numbered
        1.
    type: int
  limit:
    description:
      - Limit query parameter. The number of records
        to show for this page.
    type: int
  sortBy:
    description:
      - SortBy query parameter. A property within the
        response to sort by.
    type: str
  order:
    description:
      - Order query parameter. Whether ascending or
        descending order should be used to sort the
        response.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Sensors
      GetDeviceDeploymentStatus
    description: Complete reference of the GetDeviceDeploymentStatus
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-device-deployment-status
notes:
  - SDK Method used are
    sensors.Sensors.get_device_deployment_status,
  - Paths used are
    get /dna/intent/api/v1/icapSettings/deviceDeployments,
"""

EXAMPLES = r"""
---
- name: Get all Icap Settings Device Deployments
  cisco.dnac.icap_settings_device_deployments_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deployActivityId: string
    networkDeviceIds: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "deployActivityId": "string",
          "networkDeviceId": "string",
          "configGroupName": "string",
          "configGroupVersion": 0,
          "status": "string",
          "startTime": 0,
          "endTime": 0,
          "error": {}
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_bugs_results_network_devices_network_device_id_bugs_count_info
short_description: Information module for Network Bugs
  Results Network Devices Network Device Id Bugs Count
description:
  - Get all Network Bugs Results Network Devices Network
    Device Id Bugs Count.
  - Get count of bugs affecting the network device.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  networkDeviceId:
    description:
      - NetworkDeviceId path parameter. Id of the network
        device.
    type: str
  id:
    description:
      - Id query parameter. Id of the network bug.
    type: str
  severity:
    description:
      - Severity query parameter. Return network bugs
        with this severity. Available values CATASTROPHIC,
        SEVERE, MODERATE.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Compliance
      GetCountOfBugsAffectingTheNetworkDevice
    description: Complete reference of the GetCountOfBugsAffectingTheNetworkDevice
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-count-of-bugs-affecting-the-network-device
notes:
  - SDK Method used are
    compliance.Compliance.get_count_of_bugs_affecting_the_network_device,
  - Paths used are
    get /dna/intent/api/v1/networkBugs/results/networkDevices/{networkDeviceId}/bugs/count,
"""

EXAMPLES = r"""
---
- name: Get all Network Bugs Results Network Devices
    Network Device Id Bugs Count
  cisco.dnac.network_bugs_results_network_devices_network_device_id_bugs_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    severity: string
    networkDeviceId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "count": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: endpoint_analytics_endpoints_info
short_description: Information module for Endpoint Analytics
  Endpoints
description:
  - Get all Endpoint Analytics Endpoints.
  - Get Endpoint Analytics Endpoints by id.
  - Fetches details of the endpoint for the given unique
    identifier 'epId'. - > Query the endpoints, optionally
    using various filter and pagination criteria. 'GET
    /endpoints/count' API can be used to find out the
    total number of endpoints matching the filter criteria.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  profilingStatus:
    description:
      - >
        ProfilingStatus query parameter. Profiling status
        of the endpoint. Possible values are 'profiled',
        'partialProfiled', 'notProfiled'.
    type: str
  macAddress:
    description:
      - MacAddress query parameter. MAC address to search
        for. Partial string is allowed.
    type: str
  macAddresses:
    description:
      - MacAddresses query parameter. List of MAC addresses
        to filter on. Only exact matches will be returned.
    elements: str
    type: list
  ip:
    description:
      - Ip query parameter. IP address to search for.
        Partial string is allowed.
    type: str
  deviceType:
    description:
      - DeviceType query parameter. Type of device to
        search for. Partial string is allowed.
    type: str
  hardwareManufacturer:
    description:
      - HardwareManufacturer query parameter. Hardware
        manufacturer to search for. Partial string is
        allowed.
    type: str
  hardwareModel:
    description:
      - HardwareModel query parameter. Hardware model
        to search for. Partial string is allowed.
    type: str
  operatingSystem:
    description:
      - OperatingSystem query parameter. Operating system
        to search for. Partial string is allowed.
    type: str
  registered:
    description:
      - Registered query parameter. Flag to fetch manually
        registered or non-registered endpoints.
    type: bool
  randomMac:
    description:
      - RandomMac query parameter. Flag to fetch endpoints
        having randomized MAC or not.
    type: bool
  trustScore:
    description:
      - >
        TrustScore query parameter. Overall trust score
        of the endpoint. It can be provided either as
        a number value (e.g. 5), or as a range (e.g.
        3-7). Provide value as '-' if you want to search
        for all endpoints where trust score is not assigned.
    type: str
  authMethod:
    description:
      - AuthMethod query parameter. Authentication method.
        Partial string is allowed.
    type: str
  postureStatus:
    description:
      - PostureStatus query parameter. Posture status.
    type: str
  aiSpoofingTrustLevel:
    description:
      - >
        AiSpoofingTrustLevel query parameter. Trust
        level of the endpoint due to AI spoofing. Possible
        values are 'low', 'medium', 'high'.
    type: str
  changedProfileTrustLevel:
    description:
      - >
        ChangedProfileTrustLevel query parameter. Trust
        level of the endpoint due to changing profile
        labels. Possible values are 'low', 'medium',
        'high'.
    type: str
  natTrustLevel:
    description:
      - >
        NatTrustLevel query parameter. Trust level of
        the endpoint due to NAT access. Possible values
        are 'low', 'medium', 'high'.
    type: str
  concurrentMacTrustLevel:
    description:
      - >
        ConcurrentMacTrustLevel query parameter. Trust
        level of the endpoint due to concurrent MAC
        address. Possible values are 'low', 'medium',
        'high'.
    type: str
  ipBlocklistDetected:
    description:
      - IpBlocklistDetected query parameter. Flag to
        fetch endpoints hitting IP blocklist or not.
    type: bool
  unauthPortDetected:
    description:
      - UnauthPortDetected query parameter. Flag to
        fetch endpoints exposing unauthorized ports
        or not.
    type: bool
  weakCredDetected:
    description:
      - WeakCredDetected query parameter. Flag to fetch
        endpoints having weak credentials or not.
    type: bool
  ancPolicy:
    description:
      - AncPolicy query parameter. ANC policy. Only
        exact match will be returned.
    type: str
  limit:
    description:
      - >
        Limit query parameter. Maximum number of records
        to be fetched. If not provided, 50 records will
        be fetched by default. Maximum 1000 records
        can be fetched at a time. Use pagination if
        more records need to be fetched.
    type: int
  offset:
    description:
      - Offset query parameter. Record offset to start
        data fetch at. Offset starts at zero.
    type: int
  sortBy:
    description:
      - >
        SortBy query parameter. Name of the column to
        sort the results on. Please note that fetch
        might take more time if sorting is requested.
        Possible values are 'macAddress', 'ip'.
    type: str
  order:
    description:
      - Order query parameter. Order to be used for
        sorting. Possible values are 'asc', 'desc'.
    type: str
  include:
    description:
      - >
        Include query parameter. The datasets that should
        be included in the response. By default, value
        of this parameter is blank, and the response
        will include only basic details of the endpoint.
        To include other datasets or dictionaries, send
        comma separated list of following values 'ALL'
        - Include all attributes. 'CDP', 'DHCP', etc.
        - Include attributes from given dictionaries.
        To get full list of dictionaries, use corresponding
        GET API. 'ANC' - Include ANC policy related
        details. 'TRUST' - Include trust score details.
    type: str
  epId:
    description:
      - EpId path parameter. Unique identifier for the
        endpoint.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for AI Endpoint
      Analytics GetEndpointDetails
    description: Complete reference of the GetEndpointDetails
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-endpoint-details
  - name: Cisco DNA Center documentation for AI Endpoint
      Analytics QueryTheEndpoints
    description: Complete reference of the QueryTheEndpoints
      API.
    link: https://developer.cisco.com/docs/dna-center/#!query-the-endpoints
notes:
  - SDK Method used are
    ai_endpoint_analytics.AiEndpointAnalytics.get_endpoint_details,
    ai_endpoint_analytics.AiEndpointAnalytics.query_the_endpoints,
  - Paths used are
    get /dna/intent/api/v1/endpoint-analytics/endpoints,
    get /dna/intent/api/v1/endpoint-analytics/endpoints/{epId},
"""

EXAMPLES = r"""
---
- name: Get all Endpoint Analytics Endpoints
  cisco.dnac.endpoint_analytics_endpoints_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    profilingStatus: string
    macAddress: string
    macAddresses: []
    ip: string
    deviceType: string
    hardwareManufacturer: string
    hardwareModel: string
    operatingSystem: string
    registered: true
    randomMac: true
    trustScore: string
    authMethod: string
    postureStatus: string
    aiSpoofingTrustLevel: string
    changedProfileTrustLevel: string
    natTrustLevel: string
    concurrentMacTrustLevel: string
    ipBlocklistDetected: true
    unauthPortDetected: true
    weakCredDetected: true
    ancPolicy: string
    limit: 0
    offset: 0
    sortBy: string
    order: string
    include: string
  register: result
- name: Get Endpoint Analytics Endpoints by id
  cisco.dnac.endpoint_analytics_endpoints_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    include: string
    epId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "duid": "string",
      "macAddress": "string",
      "deviceType": [
        "string"
      ],
      "hardwareManufacturer": [
        "string"
      ],
      "hardwareModel": [
        "string"
      ],
      "operatingSystem": [
        "string"
      ],
      "lastProbeCollectionTimestamp": 0,
      "randomMac": true,
      "registered": true,
      "attributes": {},
      "trustData": {
        "trustScore": 0,
        "authMethod": "string",
        "postureStatus": "string",
        "aiSpoofingTrustLevel": "string",
        "changedProfileTrustLevel": "string",
        "natTrustLevel": "string",
        "concurrentMacTrustLevel": "string",
        "ipBlocklistDetected": true,
        "unauthPortDetected": true,
        "weakCredDetected": true
      },
      "ancPolicy": "string",
      "granularAncPolicy": [
        {
          "name": "string",
          "nasIpAddress": "string"
        }
      ]
    }
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type
from ansible.plugins.action import ActionBase

try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.dnac.plugins.plugin_utils.dnac import (
    DNACSDK,
    dnac_argument_spec,
    dnac_compare_equality,
    get_dict_result,
)
from ansible_collections.cisco.dnac.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = dnac_argument_spec()
# Add arguments specific for this module
argument_spec.update(
    dict(
        state=dict(type="str", default="present", choices=["present", "absent"]),
        tags=dict(type="list"),
        author=dict(type="str"),
        composite=dict(type="bool"),
        containingTemplates=dict(type="list"),
        createTime=dict(type="int"),
        customParamsOrder=dict(type="bool"),
        description=dict(type="str"),
        deviceTypes=dict(type="list"),
        failurePolicy=dict(type="str"),
        id=dict(type="str"),
        language=dict(type="str"),
        lastUpdateTime=dict(type="int"),
        latestVersionTime=dict(type="int"),
        name=dict(type="str"),
        parentTemplateId=dict(type="str"),
        projectId=dict(type="str"),
        projectName=dict(type="str"),
        rollbackTemplateContent=dict(type="str"),
        rollbackTemplateParams=dict(type="list"),
        softwareType=dict(type="str"),
        softwareVariant=dict(type="str"),
        softwareVersion=dict(type="str"),
        templateContent=dict(type="str"),
        templateParams=dict(type="list"),
        validationErrors=dict(type="dict"),
        version=dict(type="str"),
        templateId=dict(type="str"),
    )
)

required_if = [
    ("state", "present", ["name", "templateId"], True),
    ("state", "absent", ["name", "templateId"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class ConfigurationTemplate(object):
    def __init__(self, params, dnac):
        self.dnac = dnac
        self.new_object = dict(
            tags=params.get("tags"),
            author=params.get("author"),
            composite=params.get("composite"),
            containingTemplates=params.get("containingTemplates"),
            createTime=params.get("createTime"),
            customParamsOrder=params.get("customParamsOrder"),
            description=params.get("description"),
            deviceTypes=params.get("deviceTypes"),
            failurePolicy=params.get("failurePolicy"),
            id=params.get("id"),
            language=params.get("language"),
            lastUpdateTime=params.get("lastUpdateTime"),
            latestVersionTime=params.get("latestVersionTime"),
            name=params.get("name"),
            parentTemplateId=params.get("parentTemplateId"),
            projectId=params.get("projectId"),
            projectName=params.get("projectName"),
            rollbackTemplateContent=params.get("rollbackTemplateContent"),
            rollbackTemplateParams=params.get("rollbackTemplateParams"),
            softwareType=params.get("softwareType"),
            softwareVariant=params.get("softwareVariant"),
            softwareVersion=params.get("softwareVersion"),
            templateContent=params.get("templateContent"),
            templateParams=params.get("templateParams"),
            validationErrors=params.get("validationErrors"),
            version=params.get("version"),
            template_id=params.get("templateId"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        new_object_params["project_id"] = self.new_object.get(
            "projectId"
        ) or self.new_object.get("project_id")
        new_object_params["software_type"] = self.new_object.get(
            "softwareType"
        ) or self.new_object.get("software_type")
        new_object_params["software_version"] = self.new_object.get(
            "softwareVersion"
        ) or self.new_object.get("software_version")
        new_object_params["product_family"] = self.new_object.get(
            "productFamily"
        ) or self.new_object.get("product_family")
        new_object_params["product_series"] = self.new_object.get(
            "productSeries"
        ) or self.new_object.get("product_series")
        new_object_params["product_type"] = self.new_object.get(
            "productType"
        ) or self.new_object.get("product_type")
        new_object_params["filter_conflicting_templates"] = self.new_object.get(
            "filterConflictingTemplates"
        ) or self.new_object.get("filter_conflicting_templates")
        new_object_params["tags"] = self.new_object.get("tags")
        new_object_params["project_names"] = (
            self.new_object.get("projectName")
            or self.new_object.get("projectNames")
            or self.new_object.get("project_names")
        )
        new_object_params["un_committed"] = self.new_object.get(
            "unCommitted"
        ) or self.new_object.get("un_committed")
        new_object_params["sort_order"] = self.new_object.get(
            "sortOrder"
        ) or self.new_object.get("sort_order")
        return new_object_params

    def create_params(self):
        new_object_params = {}
        new_object_params["tags"] = self.new_object.get("tags")
        new_object_params["author"] = self.new_object.get("author")
        new_object_params["composite"] = self.new_object.get("composite")
        new_object_params["containingTemplates"] = self.new_object.get(
            "containingTemplates"
        )
        new_object_params["createTime"] = self.new_object.get("createTime")
        new_object_params["customParamsOrder"] = self.new_object.get(
            "customParamsOrder"
        )
        new_object_params["description"] = self.new_object.get("description")
        new_object_params["deviceTypes"] = self.new_object.get("deviceTypes")
        new_object_params["failurePolicy"] = self.new_object.get("failurePolicy")
        new_object_params["id"] = self.new_object.get("id")
        new_object_params["language"] = self.new_object.get("language")
        new_object_params["lastUpdateTime"] = self.new_object.get("lastUpdateTime")
        new_object_params["latestVersionTime"] = self.new_object.get(
            "latestVersionTime"
        )
        new_object_params["name"] = self.new_object.get("name")
        new_object_params["parentTemplateId"] = self.new_object.get("parentTemplateId")
        new_object_params["projectId"] = self.new_object.get("projectId")
        new_object_params["projectName"] = self.new_object.get("projectName")
        new_object_params["rollbackTemplateContent"] = self.new_object.get(
            "rollbackTemplateContent"
        )
        new_object_params["rollbackTemplateParams"] = self.new_object.get(
            "rollbackTemplateParams"
        )
        new_object_params["softwareType"] = self.new_object.get("softwareType")
        new_object_params["softwareVariant"] = self.new_object.get("softwareVariant")
        new_object_params["softwareVersion"] = self.new_object.get("softwareVersion")
        new_object_params["templateContent"] = self.new_object.get("templateContent")
        new_object_params["templateParams"] = self.new_object.get("templateParams")
        new_object_params["validationErrors"] = self.new_object.get("validationErrors")
        new_object_params["version"] = self.new_object.get("version")
        return new_object_params

    def delete_by_id_params(self):
        new_object_params = {}
        new_object_params["template_id"] = self.new_object.get("template_id")
        return new_object_params

    def update_all_params(self):
        new_object_params = {}
        new_object_params["tags"] = self.new_object.get("tags")
        new_object_params["author"] = self.new_object.get("author")
        new_object_params["composite"] = self.new_object.get("composite")
        new_object_params["containingTemplates"] = self.new_object.get(
            "containingTemplates"
        )
        new_object_params["createTime"] = self.new_object.get("createTime")
        new_object_params["customParamsOrder"] = self.new_object.get(
            "customParamsOrder"
        )
        new_object_params["description"] = self.new_object.get("description")
        new_object_params["deviceTypes"] = self.new_object.get("deviceTypes")
        new_object_params["failurePolicy"] = self.new_object.get("failurePolicy")
        new_object_params["id"] = self.new_object.get("id")
        new_object_params["language"] = self.new_object.get("language")
        new_object_params["lastUpdateTime"] = self.new_object.get("lastUpdateTime")
        new_object_params["latestVersionTime"] = self.new_object.get(
            "latestVersionTime"
        )
        new_object_params["name"] = self.new_object.get("name")
        new_object_params["parentTemplateId"] = self.new_object.get("parentTemplateId")
        new_object_params["projectId"] = self.new_object.get("projectId")
        new_object_params["projectName"] = self.new_object.get("projectName")
        new_object_params["rollbackTemplateContent"] = self.new_object.get(
            "rollbackTemplateContent"
        )
        new_object_params["rollbackTemplateParams"] = self.new_object.get(
            "rollbackTemplateParams"
        )
        new_object_params["softwareType"] = self.new_object.get("softwareType")
        new_object_params["softwareVariant"] = self.new_object.get("softwareVariant")
        new_object_params["softwareVersion"] = self.new_object.get("softwareVersion")
        new_object_params["templateContent"] = self.new_object.get("templateContent")
        new_object_params["templateParams"] = self.new_object.get("templateParams")
        new_object_params["validationErrors"] = self.new_object.get("validationErrors")
        new_object_params["version"] = self.new_object.get("version")
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method, using get all
        try:
            items = self.dnac.exec(
                family="configuration_templates",
                function="gets_the_templates_available",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if "response" in items:
                    items = items.get("response")
            result = get_dict_result(items, "name", name)
        except Exception:
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        try:
            items = self.dnac.exec(
                family="configuration_templates",
                function="get_template_details",
                params={"template_id": id},
            )
            if isinstance(items, dict):
                if "response" in items:
                    items = items.get("response")
            result = get_dict_result(items, "id", id)
        except Exception:
            result = None
        return result

    def exists(self):
        prev_obj = None
        id_exists = False
        name_exists = False
        o_id = self.new_object.get("id")
        o_id = o_id or self.new_object.get("template_id")
        name = self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            _id = _id or prev_obj.get("templateId")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters(
                    "The 'id' and 'name' params don't refer to the same object"
                )
            if _id:
                self.new_object.update(dict(id=_id))
                self.new_object.update(dict(template_id=_id))
            if _id:
                prev_obj = self.get_object_by_id(_id)
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("tags", "tags"),
            ("author", "author"),
            ("composite", "composite"),
            ("containingTemplates", "containingTemplates"),
            ("createTime", "createTime"),
            ("customParamsOrder", "customParamsOrder"),
            ("description", "description"),
            ("deviceTypes", "deviceTypes"),
            ("failurePolicy", "failurePolicy"),
            ("id", "id"),
            ("language", "language"),
            ("lastUpdateTime", "lastUpdateTime"),
            ("latestVersionTime", "latestVersionTime"),
            ("name", "name"),
            ("parentTemplateId", "parentTemplateId"),
            ("projectId", "projectId"),
            ("projectName", "projectName"),
            ("rollbackTemplateContent", "rollbackTemplateContent"),
            ("rollbackTemplateParams", "rollbackTemplateParams"),
            ("softwareType", "softwareType"),
            ("softwareVariant", "softwareVariant"),
            ("softwareVersion", "softwareVersion"),
            ("templateContent", "templateContent"),
            ("templateParams", "templateParams"),
            ("validationErrors", "validationErrors"),
            ("version", "version"),
            ("templateId", "template_id"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (ISE) params
        # If any does not have eq params, it requires update
        return any(
            not dnac_compare_equality(
                current_obj.get(dnac_param), requested_obj.get(ansible_param)
            )
            for (dnac_param, ansible_param) in obj_params
        )

    def create(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        project_id = self.new_object.get("projectId") or self.new_object.get(
            "project_id"
        )
        if not project_id:
            raise InconsistentParameters(
                "Project ID must be provided to create a template"
            )
        if not (id or name):
            raise InconsistentParameters(
                "Either 'id' or 'name' must be provided to create a template"
            )
        if id and name:
            prev_obj = self.get_object_by_name(name)
            if prev_obj:
                raise InconsistentParameters(
                    "Template with name '{}' already exists".format(name)
                )
        result = None
        result = self.dnac.exec(
            family="configuration_templates",
            function="create_template",
            params=self.create_params(),
            op_modifies=True,
        )
        return result

    def update(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        result = self.dnac.exec(
            family="configuration_templates",
            function="update_template",
            params=self.update_all_params(),
            op_modifies=True,
        )
        return result

    def delete(self):
        id = self.new_object.get("id")
        id = id or self.new_object.get("template_id")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
                id_ = id_ or prev_obj_name.get("templateId")
            if id_:
                self.new_object.update(dict(template_id=id_))
        result = self.dnac.exec(
            family="configuration_templates",
            function="deletes_the_template",
            params=self.delete_by_id_params(),
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'"
            )
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        dnac = DNACSDK(self._task.args)
        obj = ConfigurationTemplate(self._task.args, dnac)

        state = self._task.args.get("state")

        response = None
        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    dnac.object_updated()
                else:
                    response = prev_obj
                    dnac.object_already_present()
            else:
                response = obj.create()
                dnac.object_created()
        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                response = obj.delete()
                dnac.object_deleted()
            else:
                dnac.object_already_absent()

        self._result.update(dict(dnac_response=response))
        self._result.update(dnac.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wired_network_devices_network_device_id_config_features_intended_configuration_models_preview_activity_id_config_info
short_description: Information module for Wired Network
  Devices Network Device Id Config Features Intended
  Configuration Models Preview Activity Id Config
description:
  - Get Wired Network Devices Network Device Id Config
    Features Intended Configuration Models Preview Activity
    Id Config by id. - > Gets the device config for
    the configuration model. This API is 'Step 3' in
    the following workflow. Step 1- Use 'POST /intent/api/v1/wired/networkDevices/{networkDeviceId}/configFeatures/intended/configurationModels'
    to start the provision of intended features. The
    response has a taskId which is the previewActivityId
    in all subsequent APIs. The task must be successfully
    complete before proceeding to the next step. It
    is not recommended to proceed when there is any
    task failure in this step. The API 'DELETE /intent/api/v1/wired/netwo
    rkDevices/{networkDeviceId}/configFeatures/intended/configurationModels/{previewActivityId}'
    can be used at any step to discard/cancel the provision
    of intended features. Step 2- Use 'POST /intent/api/v1/wired/networkD
    evices/{networkDeviceId}/configFeatures/intended/configurationModels/{previewActivityId}/networkDevices/{netwo
    rkDeviceId}/config' to generate device CLIs for
    preview. The response has a task ID. The task must
    be successfully complete before using the GET API
    to view CLIs. It is not recommended to proceed when
    there is any task failures in this step. The API
    'DELETE /intent/api/v1/wired/networkDevices/{networkDeviceId}/configFe
    atures/intended/configurationModels/{previewActivityId}'
    can be used at any step to discard/cancel the provision
    of intended features. Step 3- Use 'GET /intent/api/v1/wired/networkDevices/{networkDeviceId}/configF
    eatures/intended/configurationModels/{previewActivityId}/networkDevices/{networkDeviceId}/config'
    to view the CLIs that will be applied to the device.
    Step 4- Use 'POST /intent/api/v1/wired/networkDevices/{networkDeviceI
    d}/configFeatures/intended/configurationModels/{previewActivityId}/deploy'
    to deploy the intent to the device.
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  networkDeviceId:
    description:
      - >
        NetworkDeviceId path parameter. Network device
        ID of the wired device to provision. The API
        /intent/api/v1/network-device can be used to
        get the network device ID.
    type: str
  previewActivityId:
    description:
      - >
        PreviewActivityId path parameter. Activity id
        is the taskId from Step 2- 'POST /intent/api/v1/wired/netw
        orkDevices/{networkDeviceId}/configFeatures/intended/configurationModels/{previewActivityId}/networkDevi
        ces/{networkDeviceId}/config.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Wired GetsTheDeviceConfigForTheConfigurationModel
    description: Complete reference of the GetsTheDeviceConfigForTheConfigurationModel
      API.
    link: https://developer.cisco.com/docs/dna-center/#!gets-the-device-config-for-the-configuration-model
notes:
  - SDK Method used are
    wired.Wired.gets_the_device_config_for_the_configuration_model,
  - Paths used are
    get /dna/intent/api/v1/wired/networkDevices/{networkDeviceId}/configFeatures/intended/configurationModels/{previewActivityId}/config,
"""

EXAMPLES = r"""
---
- name: Get Wired Network Devices Network Device Id
    Config Features Intended Configuration Models Preview
    Activity Id Config by id
  cisco.dnac.wired_network_devices_network_device_id_config_features_intended_configuration_models_preview_activity_id_config_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    networkDeviceId: string
    previewActivityId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "networkDeviceId": "string",
        "status": "string",
        "previewItems": [
          {
            "name": "string",
            "configType": "string",
            "configPreview": "string",
            "errorMessages": [
              "string"
            ]
          }
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: floors_floor_id_planned_access_point_positions_info
short_description: Information module for Floors Floor
  Id Planned Access Point Positions
description:
  - Get all Floors Floor Id Planned Access Point Positions.
  - Retrieve all Planned Access Points Positions designated
    for a specific floor.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  floorId:
    description:
      - FloorId path parameter. Floor Id.
    type: str
  name:
    description:
      - Name query parameter. Planned Access Point name.
    type: str
  macAddress:
    description:
      - MacAddress query parameter. Planned Access Point
        mac address.
    type: str
  type:
    description:
      - Type query parameter. Planned Access Point type.
    type: str
  offset:
    description:
      - Offset query parameter. The first record to
        show for this page; the first record is numbered
        1. Minimum 1.
    type: int
  limit:
    description:
      - Limit query parameter. The number of records
        to show for this page;The minimum is 1, and
        the maximum is 500.
    type: int
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Site Design
      GetPlannedAccessPointsPositionsV2
    description: Complete reference of the GetPlannedAccessPointsPositionsV2
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-planned-access-points-positions-v-2
notes:
  - SDK Method used are
    site_design.SiteDesign.get_planned_access_points_positions_v2,
  - Paths used are
    get /dna/intent/api/v2/floors/{floorId}/plannedAccessPointPositions,
"""

EXAMPLES = r"""
---
- name: Get all Floors Floor Id Planned Access Point
    Positions
  cisco.dnac.floors_floor_id_planned_access_point_positions_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    name: string
    macAddress: string
    type: string
    offset: 0
    limit: 0
    floorId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "name": "string",
          "macAddress": "string",
          "type": "string",
          "position": {
            "x": 0,
            "y": 0,
            "z": 0
          },
          "radios": [
            {
              "bands": [
                0
              ],
              "channel": 0,
              "txPower": 0,
              "antenna": {
                "name": "string",
                "azimuth": 0,
                "elevation": 0
              },
              "id": "string"
            }
          ],
          "id": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: field_notices_results_network_devices_network_device_id_info
short_description: Information module for Field Notices
  Results Network Devices Network Device Id
description:
  - Get Field Notices Results Network Devices Network
    Device Id by id.
  - Get field notice network device by device id.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  networkDeviceId:
    description:
      - NetworkDeviceId path parameter. Id of the network
        device.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Compliance
      GetFieldNoticeNetworkDeviceByDeviceId
    description: Complete reference of the GetFieldNoticeNetworkDeviceByDeviceId
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-field-notice-network-device-by-device-id
notes:
  - SDK Method used are
    compliance.Compliance.get_field_notice_network_device_by_device_id,
  - Paths used are
    get /dna/intent/api/v1/fieldNotices/results/networkDevices/{networkDeviceId},
"""

EXAMPLES = r"""
---
- name: Get Field Notices Results Network Devices Network
    Device Id by id
  cisco.dnac.field_notices_results_network_devices_network_device_id_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    networkDeviceId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "networkDeviceId": "string",
        "noticeCount": 0,
        "potentialNoticeCount": 0,
        "scanStatus": "string",
        "comments": "string",
        "lastScanTime": 0
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import functools
from typing import cast

try:
    # Beware, S3 is a "special" case, it sometimes catches botocore exceptions and
    # re-raises them as boto3 exceptions.
    import boto3
    import botocore
except ImportError:
    pass  # Handled by the calling module


from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_message
from ansible_collections.amazon.aws.plugins.module_utils.errors import AWSErrorHandler
from ansible_collections.amazon.aws.plugins.module_utils.exceptions import AnsibleAWSError

IGNORE_S3_DROP_IN_EXCEPTIONS = ["XNotImplemented", "NotImplemented", "AccessControlListNotSupported", "501"]


class AnsibleS3Error(AnsibleAWSError):
    pass


class AnsibleS3Sigv4RequiredError(AnsibleS3Error):
    pass


class AnsibleS3PermissionsError(AnsibleS3Error):
    pass


class AnsibleS3SupportError(AnsibleS3Error):
    pass


class AnsibleS3RegionSupportError(AnsibleS3SupportError):
    pass


class S3ErrorHandler(AWSErrorHandler):
    _CUSTOM_EXCEPTION = AnsibleS3Error

    @classmethod
    def _is_missing(cls):
        return is_boto3_error_code(
            [
                "404",
                "NoSuchTagSet",
                "NoSuchTagSetError",
                "ObjectLockConfigurationNotFoundError",
                "NoSuchBucketPolicy",
                "ServerSideEncryptionConfigurationNotFoundError",
                "NoSuchBucket",
                "NoSuchPublicAccessBlockConfiguration",
                "OwnershipControlsNotFoundError",
                "NoSuchOwnershipControls",
            ]
        )

    @classmethod
    def common_error_handler(cls, description):
        def wrapper(func):
            parent_class = cast(AWSErrorHandler, super(S3ErrorHandler, cls))

            @parent_class.common_error_handler(description)
            @functools.wraps(func)
            def handler(*args, **kwargs):
                try:
                    return func(*args, **kwargs)
                except is_boto3_error_code(["403", "AccessDenied"]) as e:
                    # FUTURE: there's a case to be made that this moves up into AWSErrorHandler
                    # for now, we'll handle this just for S3, but wait and see if it pops up in too
                    # many other places
                    raise AnsibleS3PermissionsError(
                        message=f"Failed to {description} (permission denied)", exception=e
                    ) from e
                except is_boto3_error_message(  # pylint: disable=duplicate-except
                    "require AWS Signature Version 4"
                ) as e:
                    raise AnsibleS3Sigv4RequiredError(
                        message=f"Failed to {description} (not supported by cloud)", exception=e
                    ) from e
                except is_boto3_error_code(IGNORE_S3_DROP_IN_EXCEPTIONS) as e:  # pylint: disable=duplicate-except
                    # Unlike most of our modules, we attempt to handle non-AWS clouds.  For read-only
                    # actions we sometimes need the ability to ignore unsupported features.
                    raise AnsibleS3SupportError(
                        message=f"Failed to {description} (not supported by cloud)", exception=e
                    ) from e
                except botocore.exceptions.EndpointConnectionError as e:
                    raise cls._CUSTOM_EXCEPTION(
                        message=f"Failed to {description} - Invalid endpoint provided", exception=e
                    ) from e
                except boto3.exceptions.Boto3Error as e:
                    raise cls._CUSTOM_EXCEPTION(message=f"Failed to {description}", exception=e) from e

            return handler

        return wrapper



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iot_fabric_rep_rings
short_description: Resource module for Iot Fabric Rep
  Rings
description:
  - Manage operation create of the resource Iot Fabric
    Rep Rings. - > This API configures a REP ring on
    FABRIC deployment. The input payload contains the
    following fields - ringName unique ring name , rootNetworkDeviceId
    Network device ID of the root node of the REP Ring
    and rootNeighbourNetworkDeviceIds Network device
    IDs of the two immediate neighbour devices of the
    root node of the REP Ring. The networkDeviceId is
    the instanceUuid attribute in the response of API
    - /dna/intent/api/v1/networkDevices.
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deploymentMode:
    description: Deployment mode of the configured REP
      ring.
    type: str
  ringName:
    description: Unique name of REP ring to be configured.
    type: str
  rootNeighbourNetworkDeviceIds:
    description: It contains the network device IDs
      of the immediate neighboring ring members of the
      root node. API `/dna/intent/api/v1/networkDevices`
      can be used to get the list of networkDeviceIds
      of the neighbors , `instanceUuid` attribute in
      the response contains rootNeighbourNetworkDeviceIds.
    elements: str
    type: list
  rootNetworkDeviceId:
    description: RootNetworkDeviceId is the network
      device ID of the root node in the REP ring. API
      `/dna/intent/api/v1/networkDevices` can be used
      to get the rootNetworkDeviceId , `instanceUuid`
      attribute in the response contains rootNetworkDeviceId.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Industrial
      Configuration ConfigureAREPRingOnFABRICDeployment
    description: Complete reference of the ConfigureAREPRingOnFABRICDeployment
      API.
    link: https://developer.cisco.com/docs/dna-center/#!configure-arep-ring-on-fabric-deployment
notes:
  - SDK Method used are
    industrial_configuration.IndustrialConfiguration.configure_a_r_e_p_ring_on_f_a_b_r_i_c_deployment,
  - Paths used are
    post /dna/intent/api/v1/iot/fabric/repRings,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.iot_fabric_rep_rings:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deploymentMode: string
    ringName: string
    rootNeighbourNetworkDeviceIds:
      - string
    rootNetworkDeviceId: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: discovery_job_info
short_description: Information module for Discovery
  Job
description:
  - Get all Discovery Job.
  - Get Discovery Job by id. - > Returns the list of
    discovery jobs for the given Discovery ID. The results
    can be optionally filtered based on IP. Discovery
    ID can be obtained using the "Get Discoveries by
    range" API.
  - Returns the list of discovery jobs for the given
    IP.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  offset:
    description:
      - Offset query parameter.
    type: int
  limit:
    description:
      - Limit query parameter. The number of records
        to show for this page. Min 1, Max 500.
    type: int
  ipAddress:
    description:
      - IpAddress query parameter.
    type: str
  name:
    description:
      - Name query parameter.
    type: str
  id:
    description:
      - Id path parameter. Discovery ID.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Discovery
      GetDiscoveryJobsByIP
    description: Complete reference of the GetDiscoveryJobsByIP
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-discovery-jobs-by-ip
  - name: Cisco DNA Center documentation for Discovery
      GetListOfDiscoveriesByDiscoveryId
    description: Complete reference of the GetListOfDiscoveriesByDiscoveryId
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-list-of-discoveries-by-discovery-id
notes:
  - SDK Method used are
    discovery.Discovery.get_discovery_jobs_by_ip,
    discovery.Discovery.get_list_of_discoveries_by_discovery_id,
  - Paths used are
    get /dna/intent/api/v1/discovery/job,
    get /dna/intent/api/v1/discovery/{id}/job,
"""

EXAMPLES = r"""
---
- name: Get all Discovery Job
  cisco.dnac.discovery_job_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    ipAddress: string
    name: string
  register: result
- name: Get Discovery Job by id
  cisco.dnac.discovery_job_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    ipAddress: string
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "attributeInfo": {},
          "cliStatus": "string",
          "discoveryStatus": "string",
          "endTime": "string",
          "httpStatus": "string",
          "id": "string",
          "inventoryCollectionStatus": "string",
          "inventoryReachabilityStatus": "string",
          "ipAddress": "string",
          "jobStatus": "string",
          "name": "string",
          "netconfStatus": "string",
          "pingStatus": "string",
          "snmpStatus": "string",
          "startTime": "string",
          "taskId": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_settings_interfaces_info
short_description: Information module for Wireless Settings
  Interfaces
description:
  - Get all Wireless Settings Interfaces.
  - Get Wireless Settings Interfaces by id.
  - This API allows the user to get all Interfaces.
  - This API allows the user to get an interface by
    ID.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  limit:
    description:
      - >
        Limit query parameter. The number of records
        to show for this page. Default is 500 if not
        specified. Maximum allowed limit is 500.
    type: float
  offset:
    description:
      - Offset query parameter. The first record to
        show for this page. The first record is numbered
        1.
    type: float
  interfaceName:
    description:
      - InterfaceName query parameter. Interface Name.
    type: str
  vlanId:
    description:
      - VlanId query parameter. Vlan Id.
    type: float
  id:
    description:
      - Id path parameter. Interface ID.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Wireless
      GetInterfaceByID
    description: Complete reference of the GetInterfaceByID
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-interface-by-id
  - name: Cisco DNA Center documentation for Wireless
      GetInterfaces
    description: Complete reference of the GetInterfaces
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-interfaces
notes:
  - SDK Method used are
    wireless.Wireless.get_interface_by_id,
    wireless.Wireless.get_interfaces,
  - Paths used are
    get /dna/intent/api/v1/wirelessSettings/interfaces,
    get /dna/intent/api/v1/wirelessSettings/interfaces/{id},
"""

EXAMPLES = r"""
---
- name: Get all Wireless Settings Interfaces
  cisco.dnac.wireless_settings_interfaces_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    limit: 0
    offset: 0
    interfaceName: string
    vlanId: 0
  register: result
- name: Get Wireless Settings Interfaces by id
  cisco.dnac.wireless_settings_interfaces_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "interfaceName": "string",
        "vlanId": 0,
        "id": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wired_network_devices_id_config_features_intended_layer2_info
short_description: Information module for Wired Network
  Devices Id Config Features Intended Layer2
description:
  - Get all Wired Network Devices Id Config Features
    Intended Layer2. - > This API returns the configurations
    for the intended layer 2 features on a wired device.
    Even after the intended configurations are deployed
    using the API /intent/api/v1/networkDevices/{id}/configFeatures/intended/deploy,
    they continue to be a part of the intended features
    on the device.
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
      - Id path parameter. Network device ID of the
        wired device to configure.
    type: str
  feature:
    description:
      - >
        Feature query parameter. Name of the feature
        to configure. The API /data/intent/api/wired/networkDevices/{id}/configFeatures/supported/layer2
        can be used to get the list of features supported
        on a device.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Wired GetConfigurationsForIntendedLayer2FeaturesOnAWiredDevice
    description: Complete reference of the GetConfigurationsForIntendedLayer2FeaturesOnAWiredDevice
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-configurations-for-intended-layer-2-features-on-a-wired-device
notes:
  - SDK Method used are
    wired.Wired.get_configurations_for_intended_layer2_features_on_a_wired_device,
  - Paths used are
    get /dna/intent/api/v1/intent/api/v1/wired/networkDevices/{id}/configFeatures/intended/layer2,
"""

EXAMPLES = r"""
---
- name: Get all Wired Network Devices Id Config Features
    Intended Layer2
  cisco.dnac.wired_network_devices_id_config_features_intended_layer2_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    feature: string
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "features": {
          "cdpGlobalConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "timer": 0,
                  "isCdpEnabled": true,
                  "isLogDuplexMismatchEnabled": true,
                  "isAdvertiseV2Enabled": true,
                  "holdTime": 0
                }
              ]
            ]
          },
          "cdpInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "interfaceName": "string",
                  "isCdpEnabled": true,
                  "isLogDuplexMismatchEnabled": true
                }
              ]
            ]
          },
          "dhcpSnoopingGlobalConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "isDhcpSnoopingEnabled": true,
                  "databaseAgent": {
                    "agentUrl": "string",
                    "timeout": 0,
                    "writeDelay": 0
                  },
                  "isGleaningEnabled": true,
                  "proxyBridgeVlans": "string"
                }
              ]
            ]
          },
          "dhcpSnoopingInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "isTrustedInterface": true,
                  "interfaceName": "string"
                }
              ]
            ]
          },
          "dot1xGlobalConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "authenticationConfigMode": "string",
                  "isDot1xEnabled": true
                }
              ]
            ]
          },
          "dot1xInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "interfaceName": "string",
                  "authenticationOrder": {
                    "configType": "string",
                    "items": [
                      "string"
                    ]
                  },
                  "priority": {
                    "configType": "string",
                    "items": [
                      "string"
                    ]
                  },
                  "inactivityTimer": 0,
                  "authenticationMode": "string",
                  "isReauthEnabled": true,
                  "maxReauthRequests": 0,
                  "isInactivityTimerFromServerEnabled": true,
                  "isReauthTimerFromServerEnabled": true,
                  "reauthTimer": 0,
                  "txPeriod": 0
                }
              ]
            ]
          },
          "igmpSnoopingGlobalConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "isIgmpSnoopingEnabled": true,
                  "isQuerierEnabled": true,
                  "querierQueryInterval": 0,
                  "querierVersion": "string",
                  "igmpSnoopingVlanSettings": {
                    "configType": "string",
                    "items": [
                      {
                        "configType": "string",
                        "vlanId": 0,
                        "isIgmpSnoopingEnabled": true,
                        "isImmediateLeaveEnabled": true,
                        "isQuerierEnabled": true,
                        "querierQueryInterval": 0,
                        "igmpSnoopingVlanMrouters": {
                          "configType": "string",
                          "items": [
                            {
                              "configType": "string",
                              "interfaceName": "string"
                            }
                          ]
                        }
                      }
                    ]
                  }
                }
              ]
            ]
          },
          "lldpGlobalConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "timer": 0,
                  "isLldpEnabled": true,
                  "reinitializationDelay": 0,
                  "holdTime": 0
                }
              ]
            ]
          },
          "lldpInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "interfaceName": "string",
                  "adminStatus": "string"
                }
              ]
            ]
          },
          "mabInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "interfaceName": "string",
                  "isMabEnabled": true
                }
              ]
            ]
          },
          "mldSnoopingGlobalConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "isMldSnoopingEnabled": true,
                  "isQuerierEnabled": true,
                  "querierQueryInterval": 0,
                  "querierVersion": "string",
                  "mldSnoopingVlanSettings": {
                    "configType": "string",
                    "items": [
                      {
                        "configType": "string",
                        "vlanId": 0,
                        "isMldSnoopingEnabled": true,
                        "isImmediateLeaveEnabled": true,
                        "isQuerierEnabled": true,
                        "querierQueryInterval": 0,
                        "mldSnoopingVlanMrouters": {
                          "configType": "string",
                          "items": [
                            {}
                          ]
                        }
                      }
                    ]
                  }
                }
              ]
            ]
          },
          "portchannelConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "isAutoEnabled": true,
                  "loadBalancingMethod": "string",
                  "lacpSystemPriority": 0,
                  "portchannels": {
                    "configType": "string",
                    "items": [
                      {
                        "configType": "string",
                        "name": "string",
                        "isLayer2": true,
                        "memberPorts": {
                          "configType": "string",
                          "items": [
                            {}
                          ]
                        }
                      }
                    ]
                  }
                }
              ]
            ]
          },
          "stpGlobalConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "stpMode": "string",
                  "isBackboneFastEnabled": true,
                  "isEtherChannelGuardEnabled": true,
                  "isExtendedSystemIdEnabled": true,
                  "isLoggingEnabled": true,
                  "isLoopGuardEnabled": true,
                  "portFastMode": "string",
                  "isBpduFilterEnabled": true,
                  "isBpduGuardEnabled": true,
                  "isUplinkFastEnabled": true,
                  "transmitHoldCount": 0,
                  "uplinkFastMaxUpdateRate": 0,
                  "stpInstances": {
                    "configType": "string",
                    "items": [
                      {
                        "configType": "string",
                        "vlanId": 0,
                        "priority": 0,
                        "isStpEnabled": true
                      }
                    ]
                  }
                }
              ]
            ]
          },
          "stpInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "interfaceName": "string",
                  "guardMode": "string",
                  "bpduFilter": "string",
                  "bpduGuard": "string",
                  "pathCost": 0,
                  "priority": 0,
                  "portVlanCostSettings": {
                    "configType": "string",
                    "items": [
                      {
                        "configType": "string",
                        "cost": 0,
                        "vlans": "string"
                      }
                    ]
                  },
                  "portVlanPrioritySettings": {
                    "configType": "string",
                    "items": [
                      {
                        "configType": "string",
                        "priority": 0,
                        "vlans": "string"
                      }
                    ]
                  }
                }
              ]
            ]
          },
          "switchportInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "interfaceName": "string",
                  "description": "string",
                  "mode": "string",
                  "accessVlan": 0,
                  "adminStatus": "string",
                  "trunkAllowedVlans": "string",
                  "nativeVlan": 0
                }
              ]
            ]
          },
          "trunkInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "interfaceName": "string",
                  "isProtected": true,
                  "isDtpNegotiationEnabled": true,
                  "pruneEligibleVlans": "string"
                }
              ]
            ]
          },
          "vlanConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "vlanId": 0,
                  "name": "string",
                  "isVlanEnabled": true
                }
              ]
            ]
          },
          "vtpGlobalConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "mode": "string",
                  "version": "string",
                  "isPruningEnabled": true,
                  "configurationFileName": "string",
                  "sourceInterface": "string"
                }
              ]
            ]
          },
          "vtpInterfaceConfig": {
            "items": [
              [
                {
                  "configType": "string",
                  "interfaceName": "string",
                  "isVtpEnabled": true
                }
              ]
            ]
          }
        }
      },
      "version": "string"
    }
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type
from ansible.plugins.action import ActionBase

try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.dnac.plugins.plugin_utils.dnac import (
    DNACSDK,
    dnac_argument_spec,
    dnac_compare_equality,
    get_dict_result,
)
from ansible_collections.cisco.dnac.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = dnac_argument_spec()
# Add arguments specific for this module
argument_spec.update(
    dict(
        state=dict(type="str", default="present", choices=["present", "absent"]),
        name=dict(type="str"),
        securityLevel=dict(type="str"),
        passphrase=dict(type="str"),
        enableFastLane=dict(type="bool"),
        enableMACFiltering=dict(type="bool"),
        trafficType=dict(type="str"),
        radioPolicy=dict(type="str"),
        enableBroadcastSSID=dict(type="bool"),
        fastTransition=dict(type="str"),
        enableSessionTimeOut=dict(type="bool"),
        sessionTimeOut=dict(type="int"),
        enableClientExclusion=dict(type="bool"),
        clientExclusionTimeout=dict(type="int"),
        enableBasicServiceSetMaxIdle=dict(type="bool"),
        basicServiceSetClientIdleTimeout=dict(type="int"),
        enableDirectedMulticastService=dict(type="bool"),
        enableNeighborList=dict(type="bool"),
        mfpClientProtection=dict(type="str"),
        nasOptions=dict(type="list"),
        profileName=dict(type="str"),
        policyProfileName=dict(type="str"),
        aaaOverride=dict(type="bool"),
        coverageHoleDetectionEnable=dict(type="bool"),
        protectedManagementFrame=dict(type="str"),
        multiPSKSettings=dict(type="list"),
        clientRateLimit=dict(type="float"),
        authKeyMgmt=dict(type="list"),
        rsnCipherSuiteGcmp256=dict(type="bool"),
        rsnCipherSuiteCcmp256=dict(type="bool"),
        rsnCipherSuiteGcmp128=dict(type="bool"),
        ghz6PolicyClientSteering=dict(type="bool"),
        ghz24Policy=dict(type="str"),
        ssidName=dict(type="str"),
    )
)

required_if = [
    ("state", "present", ["name", "ssidName"], True),
    ("state", "absent", ["name", "ssidName"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class WirelessEnterpriseSsid(object):
    def __init__(self, params, dnac):
        self.dnac = dnac
        self.new_object = dict(
            name=params.get("name"),
            securityLevel=params.get("securityLevel"),
            passphrase=params.get("passphrase"),
            enableFastLane=params.get("enableFastLane"),
            enableMACFiltering=params.get("enableMACFiltering"),
            trafficType=params.get("trafficType"),
            radioPolicy=params.get("radioPolicy"),
            enableBroadcastSSID=params.get("enableBroadcastSSID"),
            fastTransition=params.get("fastTransition"),
            enableSessionTimeOut=params.get("enableSessionTimeOut"),
            sessionTimeOut=params.get("sessionTimeOut"),
            enableClientExclusion=params.get("enableClientExclusion"),
            clientExclusionTimeout=params.get("clientExclusionTimeout"),
            enableBasicServiceSetMaxIdle=params.get("enableBasicServiceSetMaxIdle"),
            basicServiceSetClientIdleTimeout=params.get(
                "basicServiceSetClientIdleTimeout"
            ),
            enableDirectedMulticastService=params.get("enableDirectedMulticastService"),
            enableNeighborList=params.get("enableNeighborList"),
            mfpClientProtection=params.get("mfpClientProtection"),
            nasOptions=params.get("nasOptions"),
            profileName=params.get("profileName"),
            policyProfileName=params.get("policyProfileName"),
            aaaOverride=params.get("aaaOverride"),
            coverageHoleDetectionEnable=params.get("coverageHoleDetectionEnable"),
            protectedManagementFrame=params.get("protectedManagementFrame"),
            multiPSKSettings=params.get("multiPSKSettings"),
            clientRateLimit=params.get("clientRateLimit"),
            authKeyMgmt=params.get("authKeyMgmt"),
            rsnCipherSuiteGcmp256=params.get("rsnCipherSuiteGcmp256"),
            rsnCipherSuiteCcmp256=params.get("rsnCipherSuiteCcmp256"),
            rsnCipherSuiteGcmp128=params.get("rsnCipherSuiteGcmp128"),
            ghz6PolicyClientSteering=params.get("ghz6PolicyClientSteering"),
            ghz24Policy=params.get("ghz24Policy"),
            ssid_name=params.get("ssidName"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        new_object_params["ssid_name"] = self.new_object.get(
            "ssidName"
        ) or self.new_object.get("ssid_name")
        return new_object_params

    def create_params(self):
        new_object_params = {}
        new_object_params["name"] = self.new_object.get("name")
        new_object_params["securityLevel"] = self.new_object.get("securityLevel")
        new_object_params["passphrase"] = self.new_object.get("passphrase")
        new_object_params["enableFastLane"] = self.new_object.get("enableFastLane")
        new_object_params["enableMACFiltering"] = self.new_object.get(
            "enableMACFiltering"
        )
        new_object_params["trafficType"] = self.new_object.get("trafficType")
        new_object_params["radioPolicy"] = self.new_object.get("radioPolicy")
        new_object_params["enableBroadcastSSID"] = self.new_object.get(
            "enableBroadcastSSID"
        )
        new_object_params["fastTransition"] = self.new_object.get("fastTransition")
        new_object_params["enableSessionTimeOut"] = self.new_object.get(
            "enableSessionTimeOut"
        )
        new_object_params["sessionTimeOut"] = self.new_object.get("sessionTimeOut")
        new_object_params["enableClientExclusion"] = self.new_object.get(
            "enableClientExclusion"
        )
        new_object_params["clientExclusionTimeout"] = self.new_object.get(
            "clientExclusionTimeout"
        )
        new_object_params["enableBasicServiceSetMaxIdle"] = self.new_object.get(
            "enableBasicServiceSetMaxIdle"
        )
        new_object_params["basicServiceSetClientIdleTimeout"] = self.new_object.get(
            "basicServiceSetClientIdleTimeout"
        )
        new_object_params["enableDirectedMulticastService"] = self.new_object.get(
            "enableDirectedMulticastService"
        )
        new_object_params["enableNeighborList"] = self.new_object.get(
            "enableNeighborList"
        )
        new_object_params["mfpClientProtection"] = self.new_object.get(
            "mfpClientProtection"
        )
        new_object_params["nasOptions"] = self.new_object.get("nasOptions")
        new_object_params["profileName"] = self.new_object.get("profileName")
        new_object_params["policyProfileName"] = self.new_object.get(
            "policyProfileName"
        )
        new_object_params["aaaOverride"] = self.new_object.get("aaaOverride")
        new_object_params["coverageHoleDetectionEnable"] = self.new_object.get(
            "coverageHoleDetectionEnable"
        )
        new_object_params["protectedManagementFrame"] = self.new_object.get(
            "protectedManagementFrame"
        )
        new_object_params["multiPSKSettings"] = self.new_object.get("multiPSKSettings")
        new_object_params["clientRateLimit"] = self.new_object.get("clientRateLimit")
        new_object_params["authKeyMgmt"] = self.new_object.get("authKeyMgmt")
        new_object_params["rsnCipherSuiteGcmp256"] = self.new_object.get(
            "rsnCipherSuiteGcmp256"
        )
        new_object_params["rsnCipherSuiteCcmp256"] = self.new_object.get(
            "rsnCipherSuiteCcmp256"
        )
        new_object_params["rsnCipherSuiteGcmp128"] = self.new_object.get(
            "rsnCipherSuiteGcmp128"
        )
        new_object_params["ghz6PolicyClientSteering"] = self.new_object.get(
            "ghz6PolicyClientSteering"
        )
        new_object_params["ghz24Policy"] = self.new_object.get("ghz24Policy")
        return new_object_params

    def delete_by_name_params(self):
        new_object_params = {}
        new_object_params["ssid_name"] = self.new_object.get("ssid_name")
        return new_object_params

    def update_all_params(self):
        new_object_params = {}
        new_object_params["name"] = self.new_object.get("name")
        new_object_params["securityLevel"] = self.new_object.get("securityLevel")
        new_object_params["passphrase"] = self.new_object.get("passphrase")
        new_object_params["enableFastLane"] = self.new_object.get("enableFastLane")
        new_object_params["enableMACFiltering"] = self.new_object.get(
            "enableMACFiltering"
        )
        new_object_params["trafficType"] = self.new_object.get("trafficType")
        new_object_params["radioPolicy"] = self.new_object.get("radioPolicy")
        new_object_params["enableBroadcastSSID"] = self.new_object.get(
            "enableBroadcastSSID"
        )
        new_object_params["fastTransition"] = self.new_object.get("fastTransition")
        new_object_params["enableSessionTimeOut"] = self.new_object.get(
            "enableSessionTimeOut"
        )
        new_object_params["sessionTimeOut"] = self.new_object.get("sessionTimeOut")
        new_object_params["enableClientExclusion"] = self.new_object.get(
            "enableClientExclusion"
        )
        new_object_params["clientExclusionTimeout"] = self.new_object.get(
            "clientExclusionTimeout"
        )
        new_object_params["enableBasicServiceSetMaxIdle"] = self.new_object.get(
            "enableBasicServiceSetMaxIdle"
        )
        new_object_params["basicServiceSetClientIdleTimeout"] = self.new_object.get(
            "basicServiceSetClientIdleTimeout"
        )
        new_object_params["enableDirectedMulticastService"] = self.new_object.get(
            "enableDirectedMulticastService"
        )
        new_object_params["enableNeighborList"] = self.new_object.get(
            "enableNeighborList"
        )
        new_object_params["mfpClientProtection"] = self.new_object.get(
            "mfpClientProtection"
        )
        new_object_params["nasOptions"] = self.new_object.get("nasOptions")
        new_object_params["profileName"] = self.new_object.get("profileName")
        new_object_params["policyProfileName"] = self.new_object.get(
            "policyProfileName"
        )
        new_object_params["aaaOverride"] = self.new_object.get("aaaOverride")
        new_object_params["coverageHoleDetectionEnable"] = self.new_object.get(
            "coverageHoleDetectionEnable"
        )
        new_object_params["protectedManagementFrame"] = self.new_object.get(
            "protectedManagementFrame"
        )
        new_object_params["multiPSKSettings"] = self.new_object.get("multiPSKSettings")
        new_object_params["clientRateLimit"] = self.new_object.get("clientRateLimit")
        new_object_params["authKeyMgmt"] = self.new_object.get("authKeyMgmt")
        new_object_params["rsnCipherSuiteGcmp256"] = self.new_object.get(
            "rsnCipherSuiteGcmp256"
        )
        new_object_params["rsnCipherSuiteCcmp256"] = self.new_object.get(
            "rsnCipherSuiteCcmp256"
        )
        new_object_params["rsnCipherSuiteGcmp128"] = self.new_object.get(
            "rsnCipherSuiteGcmp128"
        )
        new_object_params["ghz6PolicyClientSteering"] = self.new_object.get(
            "ghz6PolicyClientSteering"
        )
        new_object_params["ghz24Policy"] = self.new_object.get("ghz24Policy")
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method or it is in another action
        try:
            items = self.dnac.exec(
                family="wireless",
                function="get_enterprise_ssid",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, list):
                for item in items:
                    if isinstance(item, dict):
                        n_item = None
                        if "response" in item:
                            n_item = item.get("response")
                        if "ssidDetails" in item:
                            n_item = item.get("ssidDetails")
                        n_item = get_dict_result(n_item, "name", name)
                        if n_item is not None:
                            return n_item
                return result
            if isinstance(items, dict):
                if "response" in items:
                    items = items.get("response")
                if "ssidDetails" in items:
                    items = items.get("ssidDetails")
            result = get_dict_result(items, "name", name)
        except Exception:
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        # NOTE: Does not have a get by id method or it is in another action
        try:
            items = self.dnac.exec(
                family="wireless",
                function="get_enterprise_ssid",
                params=self.get_all_params(id=id),
            )
            if isinstance(items, dict):
                if "response" in items:
                    items = items.get("response")
            result = get_dict_result(items, "id", id)
        except Exception:
            result = None
        return result

    def exists(self):
        id_exists = False
        name_exists = False
        prev_obj = None
        o_id = self.new_object.get("id")
        name = self.new_object.get("name")
        name = name or self.new_object.get("ssid_name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if id_exists:
            _name = prev_obj.get("name")
            _name = _name or prev_obj.get("ssidName")
            if _name:
                self.new_object.update(dict(ssid_name=_name))
        if name_exists:
            _id = prev_obj.get("id")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters(
                    "The 'id' and 'name' params don't refer to the same object"
                )
            if _id:
                self.new_object.update(dict(id=_id))
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("name", "name"),
            ("securityLevel", "securityLevel"),
            ("passphrase", "passphrase"),
            ("enableFastLane", "enableFastLane"),
            ("enableMACFiltering", "enableMACFiltering"),
            ("trafficType", "trafficType"),
            ("radioPolicy", "radioPolicy"),
            ("enableBroadcastSSID", "enableBroadcastSSID"),
            ("fastTransition", "fastTransition"),
            ("enableSessionTimeOut", "enableSessionTimeOut"),
            ("sessionTimeOut", "sessionTimeOut"),
            ("enableClientExclusion", "enableClientExclusion"),
            ("clientExclusionTimeout", "clientExclusionTimeout"),
            ("enableBasicServiceSetMaxIdle", "enableBasicServiceSetMaxIdle"),
            ("basicServiceSetClientIdleTimeout", "basicServiceSetClientIdleTimeout"),
            ("enableDirectedMulticastService", "enableDirectedMulticastService"),
            ("enableNeighborList", "enableNeighborList"),
            ("mfpClientProtection", "mfpClientProtection"),
            ("nasOptions", "nasOptions"),
            ("profileName", "profileName"),
            ("policyProfileName", "policyProfileName"),
            ("aaaOverride", "aaaOverride"),
            ("coverageHoleDetectionEnable", "coverageHoleDetectionEnable"),
            ("protectedManagementFrame", "protectedManagementFrame"),
            ("multiPSKSettings", "multiPSKSettings"),
            ("clientRateLimit", "clientRateLimit"),
            ("authKeyMgmt", "authKeyMgmt"),
            ("rsnCipherSuiteGcmp256", "rsnCipherSuiteGcmp256"),
            ("rsnCipherSuiteCcmp256", "rsnCipherSuiteCcmp256"),
            ("rsnCipherSuiteGcmp128", "rsnCipherSuiteGcmp128"),
            ("ghz6PolicyClientSteering", "ghz6PolicyClientSteering"),
            ("ghz24Policy", "ghz24Policy"),
            ("ssidName", "ssid_name"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (DNAC) params
        # If any does not have eq params, it requires update
        return any(
            not dnac_compare_equality(
                current_obj.get(dnac_param), requested_obj.get(ansible_param)
            )
            for (dnac_param, ansible_param) in obj_params
        )

    def create(self):
        result = self.dnac.exec(
            family="wireless",
            function="create_enterprise_ssid",
            params=self.create_params(),
            op_modifies=True,
        )
        return result

    def update(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        result = self.dnac.exec(
            family="wireless",
            function="update_enterprise_ssid",
            params=self.update_all_params(),
            op_modifies=True,
        )
        return result

    def delete(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        name = name or self.new_object.get("ssid_name")
        result = None
        if not name:
            prev_obj_id = self.get_object_by_id(id)
            name_ = None
            if prev_obj_id:
                name_ = prev_obj_id.get("name")
                name_ = name_ or prev_obj_id.get("ssidName")
            if name_:
                self.new_object.update(dict(ssid_name=name_))
        result = self.dnac.exec(
            family="wireless",
            function="delete_enterprise_ssid",
            params=self.delete_by_name_params(),
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'"
            )
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        dnac = DNACSDK(self._task.args)
        obj = WirelessEnterpriseSsid(self._task.args, dnac)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    dnac.object_updated()
                else:
                    response = prev_obj
                    dnac.object_already_present()
            else:
                response = obj.create()
                dnac.object_created()

        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                response = obj.delete()
                dnac.object_deleted()
            else:
                dnac.object_already_absent()

        self._result.update(dict(dnac_response=response))
        self._result.update(dnac.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_export
short_description: Resource module for Network Device
  Export
description:
  - Manage operation create of the resource Network
    Device Export.
  - Exports the selected network device to a file.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceUuids:
    description: List of device uuids.
    elements: str
    type: list
  operationEnum:
    description: 0 to export Device Credential Details
      Or 1 to export Device Details.
    type: str
  parameters:
    description: List of device parameters that needs
      to be exported to file.
    elements: str
    type: list
  password:
    description: Password is required when the operationEnum
      value is 0.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Devices
      ExportDeviceList
    description: Complete reference of the ExportDeviceList
      API.
    link: https://developer.cisco.com/docs/dna-center/#!export-device-list
notes:
  - SDK Method used are
    devices.Devices.export_device_list,
  - Paths used are
    post /dna/intent/api/v1/network-device/file,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.network_device_export:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceUuids:
      - string
    operationEnum: string
    parameters:
      - string
    password: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: task_info
short_description: Information module for Task
description:
  - Get all Task.
  - Get Task by id.
  - Returns a task by specified id.
  - Returns tasks based on filter criteria.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
      - StartTime query parameter. This is the epoch
        start time from which tasks need to be fetched.
    type: str
  endTime:
    description:
      - EndTime query parameter. This is the epoch end
        time upto which audit records need to be fetched.
    type: str
  data:
    description:
      - Data query parameter. Fetch tasks that contains
        this data.
    type: str
  errorCode:
    description:
      - ErrorCode query parameter. Fetch tasks that
        have this error code.
    type: str
  serviceType:
    description:
      - ServiceType query parameter. Fetch tasks with
        this service type.
    type: str
  username:
    description:
      - Username query parameter. Fetch tasks with this
        username.
    type: str
  progress:
    description:
      - Progress query parameter. Fetch tasks that contains
        this progress.
    type: str
  isError:
    description:
      - IsError query parameter. Fetch tasks ended as
        success or failure. Valid values true, false.
    type: str
  failureReason:
    description:
      - FailureReason query parameter. Fetch tasks that
        contains this failure reason.
    type: str
  parentId:
    description:
      - ParentId query parameter. Fetch tasks that have
        this parent Id.
    type: str
  offset:
    description:
      - Offset query parameter. The first record to
        show for this page; the first record is numbered
        1.
    type: float
  limit:
    description:
      - Limit query parameter. The number of records
        to show for this page;The minimum is 1, and
        the maximum is 500.
    type: float
  sortBy:
    description:
      - SortBy query parameter. Sort results by this
        field.
    type: str
  order:
    description:
      - Order query parameter. Sort order - asc or dsc.
    type: str
  taskId:
    description:
      - TaskId path parameter. UUID of the Task.
    type: str
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Task GetTaskById
    description: Complete reference of the GetTaskById
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-task-by-id
  - name: Cisco DNA Center documentation for Task GetTasksOperationalTasks
    description: Complete reference of the GetTasksOperationalTasks
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-tasks-operational-tasks
notes:
  - SDK Method used are
    task.Task.get_task_by_id,
    task.Task.get_tasks_operational_tasks,
  - Paths used are
    get /dna/intent/api/v1/task,
    get
    /dna/intent/api/v1/task/{taskId},
"""

EXAMPLES = r"""
---
- name: Get all Task
  cisco.dnac.task_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: string
    endTime: string
    data: string
    errorCode: string
    serviceType: string
    username: string
    progress: string
    isError: string
    failureReason: string
    parentId: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result
- name: Get Task by id
  cisco.dnac.task_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    taskId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "additionalStatusURL": "string",
        "data": "string",
        "endTime": 0,
        "errorCode": "string",
        "errorKey": "string",
        "failureReason": "string",
        "id": "string",
        "instanceTenantId": "string",
        "isError": true,
        "lastUpdate": 0,
        "operationIdList": {},
        "parentId": "string",
        "progress": "string",
        "rootId": "string",
        "serviceType": "string",
        "startTime": 0,
        "username": "string",
        "version": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: assurance_issues_resolve
short_description: Resource module for Assurance Issues
  Resolve
description:
  - Manage operation create of the resource Assurance
    Issues Resolve. - > Resolves the given list of issues.
    The response contains the list of issues which were
    successfully resolved as well as the issues which
    are failed to resolve. After this API returns success
    response, it may take few seconds for the issue
    status to be updated if the system is heavily loaded.
    Please use `GET /dna/data/api/v1/assuranceIssues/{id}`
    API to fetch the details of a particular issue and
    verify `updatedTime`. For detailed information about
    the usage of the API, please refer to the Open API
    specification document - https //github.com/cisco-en-programmability/catalyst-center-api-
    specs/blob/main/Assurance/CE_Cat_Center_Org-IssuesLifecycle-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  issueIds:
    description: Issue Ids.
    elements: str
    type: list
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Issues
      ResolveTheGivenListsOfIssues
    description: Complete reference of the ResolveTheGivenListsOfIssues
      API.
    link: https://developer.cisco.com/docs/dna-center/#!resolve-the-given-lists-of-issues
notes:
  - SDK Method used are
    issues.Issues.resolve_the_given_lists_of_issues,
  - Paths used are
    post /dna/intent/api/v1/assuranceIssues/resolve,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.assurance_issues_resolve:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: '{{my_headers | from_json}}'
    issueIds:
      - string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "successfulIssueIds": [
          "string"
        ],
        "failureIssueIds": [
          "string"
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: field_notices_results_trend_info
short_description: Information module for Field Notices
  Results Trend
description:
  - Get all Field Notices Results Trend.
  - Get field notices results trend over time. The default
    sort is by scan time descending.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  scanTime:
    description:
      - ScanTime query parameter. Return field notices
        trend with scanTime greater than this scanTime.
    type: float
  offset:
    description:
      - >
        Offset query parameter. The first record to
        show for this page; the first record is numbered
        1. Default value is 1.
    type: int
  limit:
    description:
      - >
        Limit query parameter. The number of records
        to show for this page. Minimum value is 1. Maximum
        value is 500. Default value is 500.
    type: int
requirements:
  - dnacentersdk >= 2.10.1
  - python >= 3.5
seealso:
  - name: Cisco DNA Center documentation for Compliance
      GetFieldNoticesResultsTrendOverTime
    description: Complete reference of the GetFieldNoticesResultsTrendOverTime
      API.
    link: https://developer.cisco.com/docs/dna-center/#!get-field-notices-results-trend-over-time
notes:
  - SDK Method used are
    compliance.Compliance.get_field_notices_results_trend_over_time,
  - Paths used are
    get /dna/intent/api/v1/fieldNotices/resultsTrend,
"""

EXAMPLES = r"""
---
- name: Get all Field Notices Results Trend
  cisco.dnac.field_notices_results_trend_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    scanTime: 0
    offset: 0
    limit: 0
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "scanTime": 0,
          "softwareFieldNoticesCount": 0,
          "hardwareFieldNoticesCount": 0
        }
      ],
      "version": "string"
    }
"""
